package com.jiuqi.util.patterns;

import java.util.ArrayList;
import java.util.List;

public final class Wildcard
{
    private String pattern;
    private List items;
    public static final char DEFAULT_WILDCHAR = '%';
    
    public Wildcard(final String pattern) {
        this(pattern, '%');
    }
    
    public Wildcard(final String pattern, final char wildchar) {
        this.items = new ArrayList();
        this.pattern = pattern;
        if (pattern == null || pattern.length() == 0) {
            return;
        }
        int cur = 0;
        final StringBuffer buffer = new StringBuffer(pattern.length());
        while (cur < pattern.length()) {
            final char ch = pattern.charAt(cur);
            if (ch == wildchar) {
                if (cur < pattern.length() - 1 && pattern.charAt(cur + 1) == wildchar) {
                    buffer.append(wildchar);
                    cur += 2;
                }
                else {
                    if (buffer.length() > 0) {
                        this.items.add(buffer.toString().toUpperCase());
                        buffer.setLength(0);
                    }
                    this.items.add(null);
                    ++cur;
                }
            }
            else {
                buffer.append(ch);
                ++cur;
            }
        }
        if (buffer.length() > 0) {
            this.items.add(buffer.toString().toUpperCase());
        }
    }
    
    public boolean match(final String s) {
        return s != null && this.items.size() != 0 && this.matchFrom(s.toUpperCase(), 0, 0);
    }
    
    private boolean matchFrom(final String s, int level, int fromIndex) {
        while (level < this.items.size()) {
            final String subStr = this.items.get(level);
            if (subStr != null) {
                while (fromIndex < s.length()) {
                    final int next = s.indexOf(subStr, fromIndex);
                    if (next == -1 || (level == 0 && next > 0)) {
                        return false;
                    }
                    fromIndex = next + subStr.length();
                    if (this.matchFrom(s, level + 1, fromIndex)) {
                        return true;
                    }
                }
                return false;
            }
            ++level;
        }
        return fromIndex >= s.length() || this.items.get(this.items.size() - 1) == null;
    }
    
    @Override
    public String toString() {
        return this.pattern;
    }
    
    public static boolean match(final String s, final String pattern) {
        final Wildcard wildcard = new Wildcard(pattern);
        return wildcard.match(s);
    }
    
    public static boolean match(final String s, final String pattern, final char wildchar) {
        final Wildcard wildcard = new Wildcard(pattern, wildchar);
        return wildcard.match(s);
    }
}
